#!/usr/bin/perl
# -*- mode: perl -*-

use Time::Local;
use strict;
use Jcode;

my $config = {};
my $data   = {};

my $parse = 0;
my $mode  = '';

&this_week($config);

while (<>) {
	if ($parse) {
		if (/-->/) {
			$parse = 0;
			if ($mode eq 'CONFIG') {
				&read_data($config, $data);
			} elsif ($mode eq 'TABLE') {
				&print_table($config, $data);
			} elsif ($mode eq 'CSS') {
				&print_css($config, $data);
			} elsif ($mode eq 'CATEGORY') {
				&print_category($config, $data);
			} elsif ($mode eq 'RSS') {
				&print_rss($config, $data);
			}
		} elsif (/(\w+)\s*=\s*([\w\.\/:~\+]+)/) {
			$config->{$1} = $2;
		}
	} elsif (/^<!--\#([A-Z]+)/) {
		$mode  = $1;
		$parse = 1;
	} elsif (/<title>(.+)<\/title>/) {
		$config->{title} = $1;
	} else {
		print;
	}
}

&print_rssfile ($config, $data)  if $config->{out_rss};
&print_textfile($config, $data)  if $config->{out_text};

exit;

sub this_week {
	my $config = shift;
	my $time   = localtime() + (60 * 60);

	my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($time);
	$config->{today} = {};
	$config->{today}->{date} = sprintf("%d/%d", $mon+1, $mday);
	$config->{this_week} = [];
	$config->{next_week} = [];

	for (my $i = 0; $i < 7; $i++) {
		my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($time - (24 * 60 * 60));
		$config->{this_week}->[$i] = {};
		$config->{this_week}->[$i]->{date} = sprintf("%d/%d", $mon+1, $mday);

		my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($time + (24 * 60 * 60));
		$config->{next_week}->[$i] = {};
		$config->{this_week}->[$i]->{date} = sprintf("%d/%d", $mon+1, $mday);
	}
}

sub read_data {
	my $config = shift;
	my $data   = shift;

	$config->{block_array} = [];

	$data->{blocks}      = {};
	$data->{char_hash}   = {};
	$data->{title_hash}  = {};
	$data->{match_hash}  = {};
	$data->{table_hash}  = {};
	$data->{final_hash}  = {};
	$data->{class_hash}  = {};
	$data->{class_array} = [];

	&read_tournament($config->{tournament_dat}, $data)
		or die "cannot read ".$config->{tournament_dat}."\n";
	&read_title($config->{title_dat}, $data)
		or die "cannot read ".$config->{title_dat}."\n";
	&read_characters($config->{characters_dat}, $data)
		or die "cannot read ".$config->{characters_dat}."\n";

	&do_tournament($config, $data, ['A'..'X'], 0);
	&do_tournament($config, $data, ['Z'], 1);

	select(FHOUT)  if (open(FHOUT, '>'.$config->{out_html}))
}

# CSS
sub print_css {
	my $config = shift;
	my $data   = shift;
	my $class_hash  = $data->{class_hash};
	my $class_array = $data->{class_array};

	$data->{class_columns} = &read_class($config->{css_class_dat}, $class_hash, $class_array);

	if ($config->{style_type}) {
		if ($config->{style_meta}) {
			printf('<meta http-equiv="content-style-type" content="%s">%s',
				   $config->{style_type}, "\n");
		}
		printf('<style type="%s">', $config->{style_type})   ;
	}
	print '<!--'  if ($config->{html_comment});
	print "\n"  if ($config->{style_type} || $config->{html_comment});
	foreach my $element (@{$class_hash->{'<>'}}) {
		print $element, "\n";
	}
	foreach my $key (keys(%$class_hash)) {
		next  if ($key eq '<>');
		printf(" td.%s { %s } /* %s */\n",
			   $class_hash->{$key}->{class}, $class_hash->{$key}->{css}, $key);
	}
	print '-->'  if ($config->{html_comment});
	print '</style>'  if ($config->{style_type});
	print "\n"  if ($config->{style_type} || $config->{html_comment});
}

# X^C
sub print_category {
	my $config = shift;
	my $data   = shift;
	my $class_hash  = $data->{class_hash};
	my $class_array = $data->{class_array};
	my $width = int(100 / $data->{class_columns});
	my $count = 0;

	print '<table class="tournament"><tbody><tr>', "\n";
	foreach my $category (@$class_array) {
		if ($category eq '#') {
			for (my $i = $count; $i < $data->{class_columns}; $i++) {
				print '<td class="wall">&nbsp;</td>',"\n";
			}
			print "</tr><tr>\n";
			$count = 0;
		} else {
			$count ++;
			print &td_element($config,
							  {'class' => $class_hash->{$category}->{class},
							   'style' => sprintf('width:%d%%', $width)},
							  $category), "\n";
		}
	}
	for (my $i = $count; $i < $data->{class_columns}; $i++) {
		print '<td class="wall">&nbsp;</td>',"\n";
	}
	print "</tr></tbody></table>\n";
}

# RSS
sub print_rss {
	my $config = shift;
	my $data   = shift;

	printf('<link rel="alternate" type="%s" title="%s" href="%s">'."\n",
		   $config->{rss_mimetype}, $config->{title},
		   $config->{rss_base_url}. $config->{out_rss});
}

# ΐe[u
sub print_table {
	my $config = shift;
	my $data = shift;

	&pre_make($config, $data);
	&make_table($config, $data);
}

# g[igf[^̓ǂݍ
sub read_tournament {
	my $filename   = shift;
	my $data       = shift;
	my $match_hash = $data->{match_hash};
	open(FH, $filename) or return 0;

	while (<FH>) {
		chomp;
		next  if (/^\#/);
		my @field = split(/<>/);
		my $block = shift @field;
		my $match = {};
		$match_hash->{$block} = $match;
		$match->{block} = $block;
		$match->{date}  = shift @field;
		$match->{votes} = shift @field;
		$match->{graph} = shift @field;
		$match->{alias} = shift @field;
		$match->{count} = 0;
		$match->{entry} = [];
		$match->{prev}  = [];
		if ($field[0]) {
			foreach $block (split(/,/, $field[0])) {
				if ($match_hash->{$block}) {
					$match_hash->{$block}->{next} = $match;
					push(@{$match->{prev}}, $match_hash->{$block});
				} elsif ($block =~ /^\$([A-X])(\d*)$/) {
					# ubN\
					my $final = $1;
					my $nth   = $2;
					my $entry = {};
					$nth = '0'  unless $nth;
					$final .= $nth;
					$entry->{block} = $final;
					$data->{final_hash}->{$final} = $entry;
					push(@{$match->{entry}}, $entry);
					push(@{$match->{prev}},  $entry);
					$entry->{next} = $match;
				}
			}
		}
		$match->{wday} = date2wday($config, $match->{date});
	}

	close(FH);
	return 1;
}

# if[^̓ǂݍ
sub read_title {
	my $filename   = shift;
	my $data       = shift;
	my $title_hash = $data->{title_hash};
	open(FH, $filename) or return 0;

	while (<FH>) {
		chomp;
		next  if (/^\#/);
		my @field    = split(/<>/);
		my $title    = shift @field;
		my $key      = shift @field;
		my $category = shift @field;
		$title_hash->{$key} = {};
		my $hash = $title_hash->{$key};
		$hash->{title}    = $title;
		$hash->{category} = $category;
	}

	close(FH);
	return 1;
}

# QLf[^̓ǂݍ
sub read_characters {
	my $filename   = shift;
	my $data       = shift;
	my $char_hash  = $data->{char_hash};
	my $match_hash = $data->{match_hash};
	open(FH, $filename) or return 0;

	while (<FH>) {
		chomp;
		next  if (/^\#/);
		my @field = split(/<>/);
		my $block = shift @field;
		next  unless ($block =~ /^[A-Z]\d+\-\d+$/);
		my $entry = {};
		$entry->{block}    = $block;
		$entry->{fullname} = shift @field;
		$entry->{name}     = shift @field;
		$entry->{nickname} = shift @field;
		$entry->{title}    = shift @field;
		$entry->{p_block}  = [];
		$entry->{p_vote} = [];
		$entry->{vote}   = [];
		$entry->{f_vote} = [];
		@{$entry->{p_block}} = split(/,/, shift @field);
		@{$entry->{p_vote}}  = split(/,/, shift @field);
		@{$entry->{vote}}    = split(/,/, shift @field);
		@{$entry->{f_vote}}  = split(/,/, shift @field);
		$entry->{memo}     = shift @field;

		$entry->{name}     = $entry->{fullname}  unless $entry->{name};
		$entry->{nickname} = $entry->{name}      unless $entry->{nickname};

		my $match = $match_hash->{$block};
		if ($match) {
			# ̎ɃGg[
			$match->{count} ++;
			push(@{$match->{entry}}, $entry);
			push(@{$match->{prev}},  $entry);
			$entry->{next} = $match;
		}
	}

	close(FH);
	return 1;
}

# CSS class ̓ǂݍ
sub read_class() {
	my $filename    = shift;
	my $class_hash  = shift;
	my $class_array = shift;
	my($flag, $max, $count) = (0,0,0);

	open(FH, $filename) or return 0;

	$class_hash->{'<>'} = [];
	while (<FH>) {
		chomp;
#		next  if (/^\#/);
		if (/^\#/) {
			if ($flag) {
				$flag  = 0;
				$count = 0;
				push(@$class_array, '#');
			}
			next;
		}
		my @field = split(/<>/);
		my $title = shift @field;
		my $class = shift @field;
		if ($title) {
			$class_hash->{$title} = {};
			$class_hash->{$title}->{class} = $class;
			my @style;
			foreach my $css (@field) {
				if ($css =~ /^\#[0-9A-Fa-f]{6}\b/) {
					push(@style, sprintf('background-color:%s;', $css));
				} else {
					push(@style, $css);
				}
			}
			$class_hash->{$title}->{css} = join(' ', @style);
			push(@$class_array, $title);
			$flag = 1;
			$count ++;
			$max = $count  if ($count > $max);
		} else {
			push(@{$class_hash->{'<>'}},
				 sprintf(' %s { %s }', $class, join(' ', @field)));
		}
	}
	return $max;
}

# ʏ̐
sub do_tournament {
	my $config     = shift;
	my $data       = shift;
	my $major_block_hash = shift;
	my $final_flag = shift;
	my $match_hash = $data->{match_hash};

	my $block_flag = 0;

	foreach my $major_block (@$major_block_hash) {
		$config->{step_max} = {}  unless $config->{step_max};
		$config->{game_max} = {}  unless $config->{game_max};
		$config->{total_count} = {}  unless $config->{total_count};
		foreach my $step (1..8) {
			foreach my $game (1..256) {
				my $block = sprintf("%s%d-%d", $major_block, $step, $game);
				my $match = $match_hash->{$block};
				next  unless $match;
				# g[ig\ɍڂĂ鎎

				$block_flag = 1;
				if ($config->{step_max}->{$major_block} < $step) {
					$config->{step_max}->{$major_block} = $step;
				}
				if ($config->{game_max}->{$major_block} < $game) {
					$config->{game_max}->{$major_block} = $game;
				}
				$match->{step} = $step;
				my $lv = $step - 1;
				my $array = $match->{entry};
				my $max_val = 0;
				my @winners = ();
				foreach my $entry (@$array) {
					# ō[҂̃`FbN
					if ($entry->{vote}->[$lv] == $max_val) {
						push(@winners, $entry);
					} elsif ($entry->{vote}->[$lv] > $max_val) {
						$max_val = $entry->{vote}->[$lv];
						@winners = ($entry);
					}
				}
				if ($max_val == 0) {
					$match->{max} = -1;
					next;
				}

				# ̎ɏ
				$match->{max} = $max_val;
				$match->{winners} = [];
				@{$match->{winners}} = @winners;

				# ̎ɃGg[
				if ($match->{next}) {
					push(@{$match->{next}->{entry}}, @winners);
					$match->{next}->{count} += @winners;
				} elsif (!$final_flag) {
					# g[igɃGg[
					for (my $i = 0; $i < @winners; $i++) {
						my $final = sprintf("%s%d", $major_block, $i);
						if ($data->{final_hash}->{$final}) {
							my $from = $winners[$i];
							my $to   = $data->{final_hash}->{$final};
							$to->{person}   = $from;
							$to->{name}     = $from->{fullname};
							$to->{nickname} = $from->{nickname};
							$to->{title}    = $from->{title};
							$to->{class}    = $from->{class};
							$to->{vote}     = $from->{f_vote};
							$to->{block}    = $major_block;
						}
					}
				} else {
					$match->{final} = 1;
				}
			}
		}

		if ($block_flag && !$final_flag) {
			push(@{$config->{block_array}}, $major_block);
		}
		$block_flag = 0;
	}
}

# o͏
sub pre_make() {
	my $config = shift;
	my $data   = shift;

	my $match_hash = $data->{match_hash};
	my $table_hash = $data->{table_hash};
	my $title_hash = $data->{title_hash};
	my $class_hash = $data->{class_hash};

	my $block_array = $config->{final_tournament} ? ['Z'] : $config->{block_array};

	foreach my $major_block (@$block_array) {
		$table_hash->{$major_block} = {} unless $table_hash->{$major_block};
		my $table = $table_hash->{$major_block};
		my $step = 1;
		my $game;
		my $block;
		my $match;

		$table->{count}      = [];
		$table->{count}->[0] = 0;
		my $base  = sprintf("%s%d-", $major_block, $step);
		my $total = 0;
		my @point;
		foreach $game (1..$config->{game_max}->{$major_block}) {
			$block = sprintf("%s%d", $base, $game);
			$match = $match_hash->{$block};
			next  unless $match;

			foreach my $person (@{$match->{entry}}) {
				$person->{width} = 1;

				my $category = $title_hash->{$person->{title}};
				if ($category && $category->{category} &&
					$class_hash->{$category->{category}}) {
					$person->{class} = $class_hash->{$category->{category}}->{class};
				} else {
					$person->{class} = '';
				}
			}
			$total += $match->{count};
			$table->{game_max} = $game;
			$point[$game] = $total;
		}
		$config->{total_count}->{$major_block} = $total;
		# ubN̈펎̔
		$table->{game_half}   = $table->{game_max} / 2;
		# ubNÕGg[
		$table->{count_left}  = $point[$table->{game_half}];
		# ubN㔼̃Gg[
		$table->{count_right} = $total - $table->{count_left};

		my $diff = $table->{count_left} - $table->{count_right};
		if ($config->{table_half} && ($diff > 0)) {
			# O̐l̕ = 㔼̕𒲐
			&row_span($match_hash, $table, $base, $table->{game_half}+1, $table->{game_max}, $diff);
		} elsif ($diff < 0) {
			# 㔼̐l̕ = O̕𒲐
			&row_span($match_hash, $table, $base, 1, $table->{game_half}, -$diff);
		}

		# L\ʒuvZ
		my $line = 1;
		my $half = 0;
		foreach $game (1..$config->{game_max}->{$major_block}) {
			$block = sprintf("%s%d", $base, $game);
			$match = $match_hash->{$block};
			next  unless $match;

			foreach my $person (@{$match->{entry}}) {
				$person->{line} = $line;
				$person->{half} = $half;
				$line += $person->{width};
				if ($config->{final_tournament}) {
					$person->{name}  = 'oҖ' unless $person->{name};
					$person->{title} = 'i'   unless $person->{title};
				}
			}

			if ($config->{table_half} && ($game == $table->{game_half})) {
				$line = 1;
				$half = 1;
			}
		}

		# ʕ\ʒuvZ
		foreach $step (1..$config->{step_max}->{$major_block}) {
			foreach $game (1..$config->{game_max}->{$major_block}) {
				$block = sprintf("%s%d-%d", $major_block, $step, $game);
				$match = $match_hash->{$block};
				next  unless $match;

				$match->{line}  = $match->{prev}->[0]->{line};
				$match->{half}  = $match->{prev}->[0]->{half};
				foreach my $prev (@{$match->{prev}}) {
					$match->{width} += $prev->{width};
				}
				if (!$match->{next} && $config->{table_half}) {
					$match->{width} /= 2;
				}
			}
		}

		# reLXgʕ\ʒuvZ
		my $t_pos = 0;
		foreach $game (1..$config->{game_max}->{$major_block}) {
			$block = sprintf("%s1-%d", $major_block, $game);
			$match = $match_hash->{$block};
			next  unless $match;

			$t_pos ++;
			$match->{t_top} = $t_pos;
			foreach my $person (@{$match->{entry}}) {
				$person->{t_top} = $person->{t_center} = $person->{t_bottom} = $t_pos;
				$match->{t_bottom} = $t_pos;
				$t_pos ++;
			}
			my $t_center = $match->{t_top} + $match->{t_bottom};
			$match->{t_center} = int($t_center / 2);
			if ($t_center % 2) {
				if ($game < $config->{game_max}->{$major_block} / 2) {
					$match->{t_center} ++;
				}
			}
		}
		foreach $step (2..$config->{step_max}->{$major_block}) {
			foreach $game (1..$config->{game_max}->{$major_block}) {
				$block = sprintf("%s%d-%d", $major_block, $step, $game);
				$match = $match_hash->{$block};
				next  unless $match;

				my $prev_count = @{$match->{prev}};
				$match->{t_top}    = $match->{prev}->[0]->{t_center};
				$match->{t_bottom} = $match->{prev}->[$prev_count-1]->{t_center};
				if ($prev_count % 2) {
					$match->{t_center} = $match->{prev}->[int($prev_count/2)]->{t_center};
				} else {
					$match->{t_center} = ($match->{prev}->[int($prev_count/2)-1]->{t_bottom} + $match->{prev}->[int($prev_count/2)]->{t_top}) / 2; 
				}
			}
		}
	}
}

# row_span vZ
sub row_span {
	my $match_hash = shift;
	my $table      = shift;
	my $base       = shift;
	my $begin_game = shift;
	my $end_game   = shift;
	my $diff       = shift;

	my $min = 0;
	my $match;
	my @array;
	foreach my $game ($begin_game .. $end_game) {
		my $block = sprintf("%s%d", $base, $game);
		$match = $match_hash->{$block};
		next  unless $match;

		if ($min == 0) {
			if ($match->{count}) {
				$min = $match->{count};
				@array = ($match);
			}
		} elsif ($match->{count} < $min) {
			$min = $match->{count};
			@array = ($match);
		} elsif ($match->{count} == $min) {
			push(@array, $match);
		}
	}

	foreach $match (@array) {
		last  if ($diff == 0);
		$diff --;
		my $person = $match->{entry}->[0];
		$person->{width} ++;
	}
}

# Gg[
sub entry_name {
	my $config = shift;
	my $person = shift;
	my $str = '';

	$config->{final_tournament} ?
		sprintf('<div class="from"><a href="#%s">%subN\</a></div><div class="f0">%s</div>',
				$person->{block}, $person->{block}, $person->{name}) :
		$person->{name};
}

# e[uo
sub make_table {
	my $config = shift;
	my $data   = shift;

	my $match_hash = $data->{match_hash};
	my $table_hash = $data->{table_hash};
	my $title_hash = $data->{title_hash};
	my $class_hash = $data->{class_hash};

	my($line, $attr, $content, $title, $person, $name, $count, $string, $colspan);
	my $block_array = $config->{final_tournament} ? ['Z'] : $config->{block_array};
	my $colspan = 0;

	print '<table frame="box" border="1" class="tournament"><tbody>', "\n";
	foreach my $major_block (@$block_array) {
		if ($config->{output_type} eq 'table_horizontal') {
			$colspan = $config->{step_max}->{$major_block} * 2;
			$colspan = (($colspan - 1) * 2)  if ($config->{table_half});
			$colspan++;
		}

		my $table = $table_hash->{$major_block};
		my @string_l;
		my @string_r;
		my @span;
		my @name_l;
		my @name_r;

		if ($config->{output_type} eq 'table_horizontal') {
			$string_l[0] = $string_r[0] =
				&td_element($config, {}, $config->{final_tournament} ?
							'g[ig' : sprintf('<strong>%subN</strong>', $major_block));
		}
		foreach my $step (1..$config->{step_max}->{$major_block}) {
			if ($config->{output_type} eq 'table_horizontal') {
				if ($config->{final_tournament}) {
					$string = ('', 'X', '', '')[$step];
					$string_l[0] .= &td_element($config, {'colspan' => 2}, $string);
				} elsif (($step < $config->{step_max}->{$major_block}) ||
								 !$config->{table_half}) {
					$string = &td_element($config,  {'colspan' => 2}, sprintf('%d', $step));
					$string_l[0] = $string_l[0] . $string;
					$string_r[0] = $string . $string_r[0];
				} else {
					$string_l[0] .= &td_element($config, {'style' => 'width:3em;'}, 'Block<br>');
				}
			}
			foreach my $game (1..$config->{game_max}->{$major_block}) {
				my $block = sprintf("%s%d-%d", $major_block, $step, $game);
				my $match = $match_hash->{$block};
				next  unless $match;

				# oLN^[\
				my $array = $match->{entry};
				if ($step == 1) {
					foreach $person (@$array) {
						$colspan ++  unless ($config->{output_type} eq 'table_horizontal');
						$line  = $person->{line};
						$title = $person->{title};
#						$title = $title_hash->{$title}->{title}  if ($title_hash->{$title});
						$name = &td_element($config,
											{'span'  => $person->{width},
											 'class' => $person->{class},
											 'style' => ($config->{output_type} eq 'table_vertical') ? qq(width:$config->{cell_width};) : '' },
											&entry_name($config, $person),
											sprintf('<div class="title">(%s)</div>', $title));
						if ($config->{output_type} eq 'table_horizontal') {
							if ($person->{half}) {
								$name_r[$line] = $name . "\n";
							} else {
								$name_l[$line] = $name . "\n";
							}
						} else {
							$name_l[0] .= $name;
						}
					}
				}

				# Җ\
				$line = $match->{line};
				$name = '';
				$attr = '';
				$name = &winner_element($config, $match, \$name, \$attr, $line);
				$block = $match->{alias}  if ($match->{alias});

				if (($step < $config->{step_max}->{$major_block}) ||
					!$config->{table_half}) {
					my $width = '';
					$string = &td_element($config,
										  {'span' => $match->{width},  'class' => $attr},
										  &date_element($config,
														$config->{final_tournament} ? "<strong>$block</strong>" : $block,
														$match->{date}.$match->{wday}),
										  $name);
					my $i = 0;
					my $j = 0;
					if ($match->{max} < 0) {
						# ܂sĂȂ
						foreach $person (@{$match->{prev}}) {
							my $span = $match->{prev}->[$i]->{width};
							$count = &td_element($config, {'span' => $span});
							if ($config->{output_type} eq 'table_horizontal') {
								if ($match->{half}) {
									$string_r[$line+$j] = $count . $string_r[$line+$j];
								} else {
									$string_l[$line+$j] = $string_l[$line+$j] . $count;
								}
							} else {
									$string_l[$step*2] .= $count;
								}
							$j += $span;
							$i++;
						}
					} else {
						# e҂̓[
						my $k = 0;
						my $l = 0;
						foreach $person (@$array) {
							my $span = $match->{prev}->[$i]->{width};
							if ($match->{prev}->[$i]->{winners}) {
								my $winners = @{$match->{prev}->[$i]->{winners}};
								# _P[X̔
								if ($match->{prev}->[$i]->{prev} &&
									$winners == @{$match->{prev}->[$i]->{prev}}) {
									$span = $match->{prev}->[$i]->{prev}->[$k]->{width};
								} else {
									$span = int($span / $winners)  if ($winners > 1);
								}
								$l += $span;
								$k++;
								if ($k == $winners) {
									$span += ($match->{prev}->[$i]->{width} - $l);
									$i++;
									$k = 0;
									$l = 0;
								}
							} else {
								$i++;
								$k = 0;
								$l = 0;
							}
							if ($person->{vote}->[$step-1] == $match->{max}) {
								$attr = $person->{class};
							} else {
								$attr = '';
							}
							$count = &td_element($config, {'span' => $span, 'class' => $attr},
												 $config->{final_tournament} ?
												 sprintf('<span class="count%d">%s</span>', $step, $person->{vote}->[$step-1]) :
												 $person->{vote}->[$step-1]);
							if ($config->{output_type} eq 'table_horizontal') {
								if ($match->{half}) {
									$string_r[$line+$j] = $count . $string_r[$line+$j] . "<!-- $block -->\n";
								} else {
									$string_l[$line+$j] = $string_l[$line+$j] . $count;
								}
							} else {
								$string_l[$step*2] .= $count;
							}
							$j += $span;
						}
					}
					# 
					if ($config->{output_type} eq 'table_horizontal') {
						if ($match->{half}) {
							$string_r[$line] = $string . $string_r[$line];
						} else {
							$string_l[$line] = $string_l[$line] . $string;
							for (my $i = 0; $i < $match->{width}; $i++) {
								$string_l[$line+$i] = $string_l[$line+$i] . "<!-- $block -->\n";
							}
						}
					} else {
						$string_l[$step*2+1] .= $string;
					}
				} else {
					# ubN
					if ($match->{max} < 0) {
						# ܂sĂȂ
						$string_l[$line] .= &td_element($config, {'span' => $match->{width}},
														&date_element($config, $block, $match->{date}.$match->{wday}));
					} else {
						my @left  = ();
						my @right = ();
						foreach $person (@$array) {
							if ($person->{half}) {
								push(@right, $person);
							} else {
								unshift(@left,  $person);
							}
						}
						# ʏ҂̓[
						my $i = 0;
						foreach $person (@left) {
							my $class = '';
							if ($person->{vote}->[$step-1] == $match->{max}) {
								$class = $person->{class};
							}
							$string_l[$line+$i] .= &td_element($config, {'rowspan' => 2, 'class' => $class},
															   $person->{vote}->[$step-1]);
							$i += 2;
						}
						# Eʏ҂̓[
						my $j = 0;
						foreach $person (@right) {
							my $class = '';
							if ($person->{vote}->[$step-1] == $match->{max}) {
								$class = $person->{class};
							}
							$string_l[$line+$match->{width}-2-$j] .= &td_element($config, {'rowspan' => 2, 'class' => $class},
																				 $person->{vote}->[$step-1]);
							$j += 2;
						}
						# ubN̏
						$string_l[$line+$i] .= &td_element($config, {'rowspan' => $match->{width}-$i-$j, 'class' => $attr},
														   &date_element($config, $block, $match->{date}.$match->{wday}), $name);
					}
				}
			}
		}

		print '<tr class="wall">';
		print &td_element($config, {'colspan' => $colspan, 'style' => 'text-align:left;'},
						  sprintf('<a name="%s" href="#top"></a>', $major_block));
		print "</tr>\n";
		my $max = @string_l;
		$max = @name_l  if (@name_l > $max);
		if ($config->{output_type} eq 'table_horizontal') {
			for ($line = 0; $line < $max; $line++) {
				my $class = ($line == 0) ? ' class="thead"' : '';
				print "<tr$class>\n", $name_l[$line], $string_l[$line];
				print $string_r[$line], $name_r[$line]  if $config->{table_half};
				print "</tr>\n";
			}
		} else {
			for ($line = $max-1; $line > 1; $line--) {
				print "<tr>\n", $string_l[$line], "</tr>\n";
			}
			print "<tr>\n", $name_l[0], "</tr>\n";
		}
	}
	print "</tbody></table>\n";
}

# t
sub date_element {
	my $config = shift;
	if ($config->{final_tournament}) {
		return sprintf('<div class="fdate">%s</div>', join('<br>', @_));
	} else {
		return sprintf('<span class="date">%s</span>', join(' ', @_));
	}
};

# td vf
sub td_element {
	my $config    = shift;
	my $attr_hash = shift;

	my $string = '<td';
	foreach my $attr (keys(%$attr_hash)) {
		my $value = $attr_hash->{$attr};
		next  unless $value;
		if ($attr eq 'span') {
			if ($config->{output_type} eq 'table_horizontal') {
				$attr = 'rowspan';
			} else {
				$attr = 'colspan';
			}
		}
		next  if ((($attr eq 'colspan') || ($attr eq 'rowspan')) &&
							($value == 1));
		$string .= sprintf(' %s="%s"', $attr, $value);
	}

	my $contents = join(' ', @_);
	$contents = '&nbsp;'  if ($contents =~ /^\s*$/);

	return $string . '>'. $contents . '</td>';
}

# 
sub winner_element {
	my $config   = shift;
	my $match    = shift;
	my $name_ref = shift;
	my $attr_ref = shift;
	my $delim    = 'F';
	my $begin    = '';
	my $end      = '';
	my $class    = '';

	foreach my $person (@{$match->{winners}}) {
		$$name_ref .= '^'  if $$name_ref;
		$$name_ref .= $match->{final} ? $person->{name} : $person->{nickname};
		if ($person->{class}) {
			$$attr_ref = $person->{class};
		}
	}
	return  unless $$name_ref;

	if ($config->{final_tournament}) {
		$class = 'f';
		$begin = sprintf('<div class="%s%d">', $class, $match->{step});
		$end   = '</div>';
	} else {
		if ($config->{output_type} eq 'table_horizontal') {
			if ($config->{print_vertical}) {
				$class = 'hv';
				$delim  = ' d '  if ($match->{step} >= $config->{print_vertical});
			} else {
				$class = 'hh';
			}
		} else {
			if ($config->{print_vertical}) {
				$class = 'vv';
				$delim  = ' d '  if ($match->{step} >= $config->{print_vertical});
			} else {
				$class = 'vh';
			}
		}
		$begin = sprintf('<span class="%s%d">', $class, $match->{step});
		$end   = '</span>';
	}

	if ($match->{graph}) {
		$begin = $begin . sprintf('<a href="%s%s">',
								  $config->{graph_base_url}, $match->{graph});
		$end   = '</a>' . $end;
	}
	return $begin . ($match->{final} ? 'D' : '') . $delim . $$name_ref . $end;
}

# RSS tB[h
sub print_rssfile {
	my $config = shift;
	my $data   = shift;

	return  unless open(FH, '>'.$config->{out_rss});
	select(FH);

	my $string =<<_EOH_;
<?xml version="1.0" encoding="utf-8"?>
<rss version="2.0">
 <title>$config->{title}</title>
 <link>$config->{html_base_url}</link>
 <description>$config->{description}</description>
 <language>ja</language>
_EOH_
	print Jcode->new($string)->utf8;

#  now coding...
	$string =<<_EOI_;
<item>
  <title>X/Z̎\</title>
  <link>http://etc6.2ch.net/test/read.cgi/vote/**********/</link>
  <description>1:  ~  ~ </description>
</item>

<item>
  <title>X/Y̌: 1</title>
  <link>http://```/hoge1.png</link>
  <description>: ~[ / : ~[ / : ~[</description>
</item>

<item>
  <title>X/Y̌: 2</title>
  <link>http://```/hoge2.png</link>
  <description>: ~[ / : ~[ / : ~[</description>
</item>

<item>
  <title>X/Y̌: 3</title>
  <link>http://```/hoge3.png</link>
  <description>: ~[ / : ~[ / : ~[</description>
</item>
_EOI_
#	print Jcode->new($string)->utf8;

	print <<_EOT_;
</channel>
</rss>
_EOT_

	select(STDOUT);
	close(FH);
}

# reLXgɂΐ\
sub print_textfile {
	my $config = shift;
	my $data   = shift;

	my $match_hash = $data->{match_hash};
	my $title_hash = $data->{title_hash};
	my $class_hash = $data->{class_hash};

	return  unless open(FH, '>'.$config->{out_text});
	select(FH);

	foreach my $major_block (@{$config->{block_array}}) {
		my @string;
		my @name;
		my $t_max = 0;
		foreach my $step (1..$config->{step_max}->{$major_block}) {
			my @text;
			foreach my $game (1..$config->{game_max}->{$major_block}) {
				my $block = sprintf("%s%d-%d", $major_block, $step, $game);
				my $match = $match_hash->{$block};
				next  unless $match;

				foreach my $i ($match->{t_top}..$match->{t_bottom}) {
					$text[$i] = '';
				}
				my $t_win;
				my $win_count   = 0;
				my $entry_count = @{$match->{entry}};
				my $prev_count  = @{$match->{prev}};
				my $t_center    = $match->{t_center};
				my $win_min     = $t_center;
				my $win_max     = $t_center;
				if ($match->{winners}->[0]) {
					$win_count = $match->{winners}->[0]->{vote}->[$step-1];
				}
				my $l = 0;
				my $i = 0;
				for (my $j = 0; $j < $entry_count; $j++) {
					my $t_position = $match->{prev}->[$i]->{t_center};
					my $winners = 1;
					if ($match->{prev}->[$i]->{winners}) {
						$winners = @{$match->{prev}->[$i]->{winners}};
					}
					if ($win_count &&
						$match->{entry}->[$j]->{vote}->[$step-1] == $win_count) {
						$t_win = $t_position;
						if ($t_position < $t_center) {
							$text[$t_position] = '';
							if ($win_min > $t_position) {
								$win_min = $t_position;
							}
						} elsif ($t_position > $t_center) {
							$text[$t_position] = '';
							if ($win_max < $t_position) {
								$win_max = $t_position;
							}
						} else {
							$text[$t_position] = '';
						}
					} else {
						if ($i == 0) {
							$text[$t_position] = '';
						} elsif ($i == $prev_count-1) {
							$text[$t_position] = '';
						} else {
							$text[$t_position] = '';
						}
					}
					$l ++;
					if ($l == $winners) {
						$i ++;
						$l = 0;
					}
					if ($step == 1) {
						my $title = $match->{entry}->[$j]->{title};
						if ($data->{title_hash}->{$title}) {
							$title = $data->{title_hash}->{$title}->{title};
						}
						$name[$t_position] = $match->{entry}->[$j]->{fullname} . '' . $title;
						$t_max = $t_position  if ($t_max < $t_position);
					}
				}
				if ($text[$t_center] eq '') {
					$text[$t_center] = '';
				} elsif ($text[$t_center] eq '') {
					$text[$t_center] = '';
				}

				if ($win_count) {
					if ($t_win < $match->{t_center}) {
						$text[$t_center] = '';
					} elsif ($match->{t_center} < $t_win) {
						$text[$t_center] = '';
					}
					foreach my $position ($win_min+1..$win_max-1) {
						if ($position == $t_center) {
							$text[$position] = '';
						} else {
							$text[$position] = '';
						}
					}
				}
			}
			foreach my $i (1..$t_max) {
				$string[$i] = ($text[$i] ? $text[$i] : '@') . $string[$i];
			}
		}
		printf("\n[%subN]\n", $major_block);
		foreach my $i (1..$t_max) {
			printf("%s %s\n", $string[$i], $name[$i]);
		}
	}

	select(STDOUT);
	close(FH);
}

# tj߂
sub date2wday {
	my $config = shift;
	my $date   = shift;

	return  unless $date;
	my ($mon, $mday) = split(/\//, $date);

	return  unless $config->{start_date};
	unless ($config->{start_year}) {
		my ($start_year, $start_mon, $start_mday) = split(/\//, $config->{start_date});
		if ($start_year > 2000) {
			$config->{start_year} = $start_year;
		}
		if (1 <= $start_mon && $start_mon <= 12) {
			$config->{start_mon}  = $start_mon;
		}
		if ($start_mday < 32) {
			$config->{start_mday} = $start_mday;
		}
	}

	my $year = $config->{start_year};
	$year++  if ($mon < $config->{start_mon});	# NzƔf

	my $wday = (localtime(timelocal(0, 0, 0, $mday, $mon-1, $year)))[6];
	return ('()', '()', '()', '()', '()', '()', '(y)')[$wday];
}

__END__

	Unlimited Script Works 3

# Local Variables:
# tab-width:4
# End:
